//gamewindow.cpp - definition of the methods of GameWindow
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gamewindow.h>

//-------------------------------------------------------------
/*
 * constructor of GameWindow
 */

GameWindow::GameWindow()
{
  main_window_ = new GnuzWindow(TRUE);
  levelwidget_ = new QWidget(main_window_);
  config_ = new GnuzConfig;

  guipics_ = new GuiPics(config_);
  gnutypeselector_ = new GnuTypeSelector(levelwidget_, config_, guipics_);
  levelloader_ = new LevelLoader(config_, guipics_);
  level_ = new Level(levelwidget_, gnutypeselector_, config_, guipics_);
  level_->loadLevel(config_->getLevelDir()+config_->getIntro());

  QObject::connect(main_window_, SIGNAL(keyPressEventSignal(QKeyEvent*)), this, SLOT(handleKeyPressedEvent(QKeyEvent*)));
  QObject::connect(main_window_, SIGNAL(windowclosed()), config_, SLOT(writeConfig()));
  QObject::connect(gnutypeselector_, SIGNAL(newType(QString)), level_->getHerd(), SLOT(changeType(QString)));
  QObject::connect(level_->getHerd(), SIGNAL(changeButtonPixmap(QString, QString)), gnutypeselector_, SLOT(changeButtonPixmap(QString, QString)));
  QObject::connect(level_->getLevelCanvasView(), SIGNAL(GnuTypeChanged()), gnutypeselector_, SLOT(GnuTypeChanged()));
  QObject::connect(gnutypeselector_->getGnusLeftCounter(), SIGNAL(valueChanged()), level_, SLOT(GnusLeftChanged()));
  QObject::connect(level_, SIGNAL(setButtonsDisabled(bool)), this, SLOT(setButtonsDisabled(bool)));
  QObject::connect(level_, SIGNAL(hideLevelLoader()), this, SLOT(hideLevelLoader()));
  QObject::connect(levelloader_, SIGNAL(loadLevel(QString)), level_, SLOT(loadLevel(QString)));
  QObject::connect(levelloader_, SIGNAL(cancelLoadLevel()), level_, SLOT(cancelLoadLevel()));

  main_layout_ = new QVBoxLayout(levelwidget_);
  
  main_layout_->addWidget(level_->getLevelView());
  main_layout_->addLayout(gnutypeselector_->getLayout());
  main_layout_->insertStretch(-1);

  gnutypeselector_->getLayout()->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
  
  main_window_->setCaption("gnuZ");
  main_window_->setCentralWidget(levelwidget_);
 
  filemenu_ = new QPopupMenu;
  filemenu_->insertItem(*guipics_->getPic("NavLoadLevel"), "&Load Level...", this, SLOT(openLevelLoader()));
  filemenu_->insertItem(*guipics_->getPic("NavRestartLevel"), "&Restart Level", level_, SLOT(restartLevel()));
  filemenu_->insertItem(*guipics_->getPic("NavQuit"), "&Quit", qApp, SLOT(quit()));
  helpmenu_ = new QPopupMenu;
  helpmenu_->insertItem(*guipics_->getPic("NavAbout"), "&About", this, SLOT(displayAboutWindow()));

  menubar_ = new QMenuBar(main_window_);
  menubar_->insertItem("&File", filemenu_);
  menubar_->insertItem("&Help", helpmenu_);

  setButtonsDisabled(TRUE);

  main_window_->showMaximized();
  main_window_->show();
}

//-------------------------------------------------------------
/*
 * destructor of GameHandler
 */

GameWindow::~GameWindow()
{
  delete level_;
  delete levelwidget_;
  delete gnutypeselector_;
  delete main_layout_;
  delete filemenu_;
  delete helpmenu_;
  delete menubar_;
  delete levelloader_;
  delete main_window_;
  delete config_;
}

//-------------------------------------------------------------
/*
 * SLOT: change disabled status of all buttons (GnuTypeSelector)
 */

void GameWindow::setButtonsDisabled(bool disabled)
{
  scrolling_disabled_=disabled;

  gnutypeselector_->setButtonsDisabled(disabled);
}

//-------------------------------------------------------------
/*
 * SLOT: Open LevelLoader window and stop game
 */

void GameWindow::openLevelLoader()
{
  level_->setAdvancePeriod(-1); //Stop game in the background

  if (!levelloader_->isWindowCreated())
    levelloader_->createWindow();

  levelloader_->show();
  levelloader_->raise();
}

//-------------------------------------------------------------
/*
 * SLOT: Hide Level Loader
 */

void GameWindow::hideLevelLoader()
{
  levelloader_->hide();
}

//-------------------------------------------------------------
/*
 * SLOT: About Window
 */

void GameWindow::displayAboutWindow()
{
  QString about="gnuZ 0.3\n\nVisit http://gnuz.4cows.net\n\n(c) 2003 - Lukas Fraser\n<lukas.fraser@4cows.net>\n\nLicensed under the GPL";
  QMessageBox::information(main_window_, "About", about.latin1());
}

//-------------------------------------------------------------
/*
 * SLOT: Handle KeyPressed Events
 *
 * @param e keypressed event
 */

void GameWindow::handleKeyPressedEvent(QKeyEvent* e)
{
  switch(e->key())
  {
    case Qt::Key_Left:
    {
      if (!scrolling_disabled_)
	level_->getLevelScrollBar()->subtractLine();
      break;
    }
    
    case Qt::Key_Right:
    {
      if (!scrolling_disabled_)
	level_->getLevelScrollBar()->addLine();
      break;
    }
  }
}
