//herd.cpp - definition of the methods of Herd
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <herd.h>

//-------------------------------------------------------------
/*
 * constructor of Herd
 *
 * @param parent canvas
 * @param levelpics the level pictures (mask + pic)
 * @param gnusleft label for counting living gnus
 * @param gnussaved label for counting saved gnus
 */

Herd::Herd(QCanvas *parent, int *level_advanceperiod, LevelPics *levelpics, GnuTypeSelectLabel *gnusleft, GnuTypeSelectLabel *gnussaved, GnuzConfig *config)
{
  config_=config;
  gnupics_ = new GnuPics(config_);

  level_advanceperiod_=level_advanceperiod;

  parent_=parent;
  levelpics_=levelpics;
  type_="Walker";
  gnusleft_=gnusleft;
  gnussaved_=gnussaved;
  speed_=config_->getNormalGnuSpeed();
}

//-------------------------------------------------------------
/*
 * destructor of Herd
 */

Herd::~Herd()
{
  delete gnupics_;
  delete garbage_canvas_;
}

//-------------------------------------------------------------
/*
 * SLOT: change active Gnu Type; pause game / kill all Gnus
 *
 * @param type new selected type (pressed button)
 */

void Herd::changeType(QString type)
{
  bool temp;

  if (type=="KillAll") //Kill 'em all -> turn all Gnus to Bombers
  {
    type_="Bomber";
    for (int i=0;i<(int)herd_.count();i++)
    {
      if (herd_[i]->active())
      {
	herd_[i]->setExplodeCountdown(100+(rand() % 50));
	temp=herd_[i]->clicked();
      }

      emit changeButtonPixmap("KillAll", "Bomb_activated");
    }
    return;
  }

  if (type=="Pause") //Pause
  {
    if (*level_advanceperiod_>=0)
    {
      emit changeAdvancePeriod(-1);
      emit changeButtonPixmap("Pause", "Play");
    }
    else
    {
      emit changeAdvancePeriod(speed_);
      emit changeButtonPixmap("Pause", "Pause");
    }
    return;
  }

  if (type=="FastForward") //FastForward
  {
    if (fast_mode_)
    {
      for (int i=0;i<(int)herd_.count();i++)
      {
	if (herd_[i]->active())
	  herd_[i]->setFastMode(FALSE);
      }

      fast_mode_=FALSE;
      emit changeFastForwardMode(fast_mode_);
      emit changeButtonPixmap("FastForward", "FastForward_inactive");
    }
    else
    {
      for (int i=0;i<(int)herd_.count();i++)
      {
	if (herd_[i]->active())
	  herd_[i]->setFastMode(TRUE);
      }

      fast_mode_=TRUE;
      emit changeFastForwardMode(fast_mode_);
      emit changeButtonPixmap("FastForward", "FastForward");
    }
    return;
  }

  type_=type;
}

//-------------------------------------------------------------
/*
 * Create a Gnu Herd
 *
 * @param gnu_count number of Gnus
 * @param gnu_xspawn
 * @param gnu_yspawn
 * @param gnu_orientation (0=left, 1=right)
 */

void Herd::createHerd(int gnu_count, int gnu_xspawn, int gnu_yspawn, int gnu_orientation)
{
  gnu_count_=gnu_count;
  speed_=config_->getNormalGnuSpeed();

  emit changeAdvancePeriod(speed_);
  fast_mode_=FALSE;

  if ((int)herd_.count()<gnu_count) //Enlarge herd size
  {
    //printf("Enlarging Herd %i - %i\n", (int)herd_.count(), gnu_count);
    for (int i=(int)herd_.count();i<gnu_count;i++)
    {
      herd_.append(new Gnu(gnupics_, levelpics_, parent_, &type_, config_));
      herd_[i]->setGnusLeftCounter(gnusleft_);
      herd_[i]->setGnusSavedCounter(gnussaved_);
    }
    //printf("Enlarged Herd %i - %i\n", (int)herd_.count(), gnu_count);
  }

  if ((int)herd_.count()>gnu_count) //Remove unused gnus
  {
    //printf("Removing Gnus %i - %i\n", (int)herd_.count(), gnu_count);
    for (int i=(int)herd_.count();i>gnu_count;i--)
    {
      herd_[i-1]->setVisible(FALSE);
      herd_[i-1]->setActive(FALSE);
      delete herd_[i-1];
      herd_.remove(herd_[i-1]);
    }
    //printf("Removed Gnus %i - %i\n", (int)herd_.count(), gnu_count);
  }

  for (int i=0;i<gnu_count;i++) //Set params for herd
  {
    herd_[i]->resetGnu();

    if ((gnu_xspawn>=0)&&(gnu_xspawn+gnupics_->getGnuWidth()<levelpics_->getLevelWidth()))
    {
      if (gnu_yspawn>=0)
	herd_[i]->setSpawningInsideLevel(TRUE, gnu_yspawn);
      herd_[i]->move(gnu_xspawn, gnu_yspawn-((gnupics_->getGnuHeight()+7)*(i+1)));
      herd_[i]->setSkin("Fall");
    }

    if (gnu_xspawn<0)
    {
      gnu_orientation=1;
      herd_[i]->move(gnu_xspawn-((gnupics_->getGnuWidth()+3)*(i+1)), gnu_yspawn);
    }

    if (gnu_xspawn+gnupics_->getGnuWidth()>levelpics_->getLevelWidth())
    {
      gnu_orientation=0;
      herd_[i]->move(gnu_xspawn+((gnupics_->getGnuWidth()+3)*i), gnu_yspawn);
    }

    herd_[i]->setOrientation(gnu_orientation);
    herd_[i]->setZ(gnu_count-i);
  }
}
