//level.cpp - definition of the methods of Level
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <level.h>

//-------------------------------------------------------------
/*
 * constructor of Level
 *
 * @param parent
 * @param gnutypeselector
 */

Level::Level(QWidget *parent, GnuTypeSelector *gnutypeselector, GnuzConfig *config, GuiPics *guipics)
{
  parent_widget_=parent;
  gnutypeselector_=gnutypeselector;
  config_=config;
  guipics_=guipics;

  gnuztimer_ = new GnuzTimer();

  levelconfig_ = new LevelConfig();
  levelstats_ = new LevelStats(gnutypeselector_, levelconfig_, config_, guipics_, gnuztimer_);
  levelpics_ = new LevelPics(config_);
  level_ = new QCanvas(parent);

  levelpic_ = new QCanvasSprite(levelpics_->getLevelCanvasPixmapArray(), level_);

  levelpics_->setLevelSprite(levelpic_);

  level_view_ = new LevelCanvasView(level_, parent);
  level_view_->setMinimumHeight(config_->getHeight());
  level_view_->setMaximumHeight(config_->getHeight());
  level_view_->setHScrollBarMode(QScrollView::AlwaysOff);

  level_view_->setLineWidth(0);
  level_view_->horizontalScrollBar()->setLineStep(40);

  herd_ = new Herd(level_, &level_advanceperiod_, levelpics_, gnutypeselector_->getGnusLeftCounter(), gnutypeselector_->getGnusSavedCounter(), config_);

  QObject::connect(herd_, SIGNAL(changeAdvancePeriod(int)), this, SLOT(setAdvancePeriod(int)));
  QObject::connect(herd_, SIGNAL(changeFastForwardMode(bool)), this, SLOT(setFastForwardMode(bool)));
}

//-------------------------------------------------------------
/*
 * destructor of Level
 */

Level::~Level()
{
  delete levelstats_;
  delete levelconfig_;
  delete levelpic_;
  delete levelpics_;
  delete level_;
  delete level_view_;
  delete herd_;
}

//-------------------------------------------------------------
/*
 * SLOT: Gnus left changed, check if game is over. If yes -> stop game and display stats
 */

void Level::GnusLeftChanged()
{
  if (!herd_->getGnusLeftCounter()->getNum())
  {
    emit setButtonsDisabled(1);
    level_view_->horizontalScrollBar()->setValue(0);
    setAdvancePeriod(-1);
    levelpic_->setSequence(levelstats_->getStats());
  }
}

//-------------------------------------------------------------
/*
 * SLOT: Restart current level
 */

void Level::restartLevel()
{
  if (current_level_!=config_->getLevelDir()+"intro")
    loadLevel(current_level_);
}

//-------------------------------------------------------------
/*
 * SLOT: set AdvancePeriod of level (Pause)
 */

void Level::setAdvancePeriod(int period)
{
  level_->setAdvancePeriod(period);

  if (period<0)
    gnuztimer_->pause();
  else
    gnuztimer_->resume();

  level_advanceperiod_=period;
} 

//-------------------------------------------------------------
/*
 * SLOT: Activate/Deactivate FF Mode
 *
 * @param mode
 */

void Level::setFastForwardMode(bool active)
{
  if (active)
    gnuztimer_->setFastForward(2);
  else
    gnuztimer_->setFastForward(1);
}

//-------------------------------------------------------------
/*
 * Load Level
 *
 * @param filename level .conf file
 */

void Level::loadLevel(QString filename)
{
  gnuztimer_->restart();
  current_level_=filename;
  levelconfig_->loadConfig(filename);

  gnutypeselector_->setGnuConfig(levelconfig_->getGnuData("GnuCount"), levelconfig_->getGnuData("GnuMinSave"), levelconfig_->getGnuData("GnuBlockers"), levelconfig_->getGnuData("GnuDiggers"), levelconfig_->getGnuData("GnuBashers"), levelconfig_->getGnuData("GnuBuilders"), levelconfig_->getGnuData("GnuFloaters"), levelconfig_->getGnuData("GnuClimbers"), levelconfig_->getGnuData("GnuBombers"));

  levelpics_->loadPics(levelconfig_->getLevelPic(), levelconfig_->getLevelMask());

  level_->resize(levelpics_->getLevelWidth(), levelpics_->getLevelHeight());

  herd_->createHerd(levelconfig_->getGnuData("GnuCount"), levelconfig_->getGnuData("GnuXSpawnPoint"), levelconfig_->getGnuData("GnuYSpawnPoint"), levelconfig_->getGnuData("GnuOrientation"));

  levelpic_->setSequence(levelpics_->getLevelCanvasPixmapArray());
  levelpic_->show();

  level_view_->horizontalScrollBar()->setValue(levelconfig_->getGnuData("GnuXSpawnPoint")-90); //FIXXME

  emit setButtonsDisabled(FALSE);
  emit updateScrollButtons();
}

//-------------------------------------------------------------
/*
 * cancel LevelLoader and return to current level
 */

void Level::cancelLoadLevel()
{
  setAdvancePeriod(herd_->getSpeed());
}
