//levelloader.cpp - definition of the methods of LevelLoader
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <levelloader.h>

//-------------------------------------------------------------
/*
 * constructor of LevelLoader
 */
LevelLoader::LevelLoader(GnuzConfig *config, GuiPics *guipics)
{
  config_=config;
  guipics_=guipics;

  window_created_=FALSE;
}

//-------------------------------------------------------------
/*
 * Create window and Load level infos
 */

void LevelLoader::createWindow()
{
  loader_window_ = new GnuzWindow();
  loader_widget_ = new QWidget(loader_window_);

  loader_window_->setCaption("Load Level...");
  loader_window_->setCentralWidget(loader_widget_);

  loader_layout_ = new QVBoxLayout(loader_widget_);
  button_layout_ = new QHBoxLayout(0);

  levellist_ = new QListView(loader_widget_);
  levellist_->addColumn("Filename", 0);
  levellist_->addColumn("Title", 140);
  levellist_->addColumn("Diff");
  levellist_->addColumn("Gnus");
  levellist_->addColumn("Block");
  levellist_->addColumn("Dig");
  levellist_->addColumn("Bash");
  levellist_->addColumn("Build");
  levellist_->addColumn("Float");
  levellist_->addColumn("Climb");
  levellist_->addColumn("Bomb");

  for (int i=3;i<=11;i++)
    levellist_->setColumnAlignment(i, Qt::AlignHCenter);

  levellist_->setAllColumnsShowFocus(1);
  //levellist_->setShowSortIndicator(1);
  levellist_->setFocus();

  QObject::connect(levellist_, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(updatePreviewPic(QListViewItem *)));
  QObject::connect(levellist_, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(loadLevelclicked()));

  levelpic_ = new QLabel(loader_widget_);
  levelpic_->setFixedSize(240, 80);
  levelpic_->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
  
  loadbutton_ = new QPushButton(*guipics_->getPic("LevelLoader_OK"), "Load Level", loader_widget_);
  cancelbutton_ = new QPushButton(*guipics_->getPic("LevelLoader_Cancel"), "Cancel", loader_widget_);
  button_layout_->addWidget(loadbutton_);
  button_layout_->addWidget(cancelbutton_);
  QObject::connect(loadbutton_, SIGNAL(clicked()), this, SLOT(loadLevelclicked()));
  QObject::connect(cancelbutton_, SIGNAL(clicked()), this, SLOT(cancelclicked()));
  QObject::connect(loader_window_, SIGNAL(windowclosed()), this, SLOT(cancelclicked()));

  loader_layout_->addWidget(levellist_);
  loader_layout_->addWidget(levelpic_);
  loader_layout_->addLayout(button_layout_);

  levelconfig_ = new LevelConfig();

  getLevels();

  loader_window_->showMaximized();
  loader_window_->hide();

  window_created_=TRUE;
}

//-------------------------------------------------------------
/*
 * destructor of LevelLoader
 */

LevelLoader::~LevelLoader()
{
  delete levellist_;
  delete levelpic_;
  delete loadbutton_;
  delete cancelbutton_;
  delete button_layout_;
  delete loader_layout_;
  delete loader_widget_;
  delete loader_window_;
}

//-------------------------------------------------------------
/*
 * SLOT: Load selected level and close window
 */

void LevelLoader::loadLevelclicked()
{
  if (levellist_->childCount())
  {
    loader_window_->hide();
    emit loadLevel(config_->getLevelDir()+levellist_->selectedItem()->text(0));
  }
}

//-------------------------------------------------------------
/*
 * SLOT: cancel and close window
 */

void LevelLoader::cancelclicked()
{
  loader_window_->hide();
  emit cancelLoadLevel();
}

//-------------------------------------------------------------
/*
 * SLOT: Update Preview Pic
 *
 * @param item selected item
 */

void LevelLoader::updatePreviewPic(QListViewItem * item)
{
  double new_width=levelpic_->width();
  double new_height=levelpic_->height();
  levelconfig_->loadConfig(config_->getLevelDir()+item->text(0));
  QImage preview(levelconfig_->getLevelPic().convertToImage());
  double original_width=preview.width();
  double original_height=preview.height();

  if (original_width>levelpic_->width())
  {
    new_width=levelpic_->width();
    new_height=original_height*(new_width/original_width);

    if (new_height>levelpic_->height())
    {
      new_height=levelpic_->height();
      new_width=original_width*(new_height/original_height);
    }
  }

  preview=preview.smoothScale((int)new_width, (int)new_height);
  QPixmap preview_pixmap;
  preview_pixmap=preview;

  levelpic_->setPixmap(preview_pixmap);
}

//-------------------------------------------------------------
/*
 * Load Levels and put them into levellist if they are valid
 */

void LevelLoader::getLevels()
{
  QDir leveldir(config_->getLevelDir());
  if (!leveldir.exists())
    qWarning("Cannot find the level directory");
  else
  {
    leveldir.setFilter(QDir::Files);
    leveldir.setSorting(QDir::Name | QDir::Reversed);
    leveldir.setNameFilter("*.lev");

    const QFileInfoList *list = leveldir.entryInfoList();
    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    while ((fi = it.current()) != 0)
    {      
      levelconfig_->loadConfig(config_->getLevelDir()+fi->fileName(), TRUE);
      if (levelconfig_->isValid())
      {
	LevelLoaderItem* item = new LevelLoaderItem(levellist_, fi->fileName(), levelconfig_->getLevelData("LevelName"), levelconfig_->getLevelData("LevelDifficulty"), levelconfig_->getLevelData("GnuCount")+"("+levelconfig_->getLevelData("GnuMinSave")+")", levelconfig_->getLevelData("GnuBlockers"), levelconfig_->getLevelData("GnuDiggers"), levelconfig_->getLevelData("GnuBashers"), levelconfig_->getLevelData("GnuBuilders"), levelconfig_->getLevelData("GnuFloaters"), levelconfig_->getLevelData("GnuClimbers"), levelconfig_->getLevelData("GnuBombers"));
	
	QString difficulty;
	
	switch(levelconfig_->getLevelData("LevelDifficulty").toInt())
	{
	case 1:  { difficulty="1"; break; }
	case 2:  { difficulty="2"; break; }
	case 3:  { difficulty="3"; break; }
	case 4:  { difficulty="4"; break; }
	default: { difficulty="0"; break; }
	}
	if (!difficulty.isEmpty())
	{
	  item->setPixmap(2, *guipics_->getPic("Difficulty"+difficulty));
	  item->setDifficulty(difficulty);
	}
	item->setText(2, "");
      }
      ++it;
    }
    if (levellist_->firstChild())
    {
      levellist_->setSelected(levellist_->firstChild(), 1);
      updatePreviewPic(levellist_->firstChild());
    }
  }
}
