//levelpics.cpp - definition of the methods of LevelPics
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <levelpics.h>

//-------------------------------------------------------------
/*
 * constructor of LevelPics
 */

LevelPics::LevelPics(GnuzConfig *config)
{
  config_=config;
  levelcanvaspixmaparray_ = new QCanvasPixmapArray();
}

//-------------------------------------------------------------
/*
 * destructor of LevelPics
 */

LevelPics::~LevelPics()
{
  delete levelmask_;
  delete levelcanvaspixmap_;
  delete levelcanvaspixmaparray_;
  delete levelsprite_;
}

//-------------------------------------------------------------
/*
 * load pics
 *
 * @param levelpicfilename
 * @param levelmaskfilename
 */

void LevelPics::loadPics(QPixmap levelpicfile, QImage levelmaskfile)
{
  levelmask_ = new QImage(levelmaskfile);
  levelmask_converted_ = levelmask_->convertDepth(32, QPixmap::Auto);
  levelmask_ = &levelmask_converted_;

  QList<QPixmap> picslist;
  QList<QPoint> pointlist;
  QPoint temppoint;
  
  picslist.append(&levelpicfile);
  pointlist.append(&temppoint);

  levelcanvaspixmaparray_ = new QCanvasPixmapArray(picslist, pointlist);
  levelcanvaspixmap_ = levelcanvaspixmaparray_->image(0);
}

//-------------------------------------------------------------
/*
 * Update LevelCanvasPixmapArray (repaint)
 */

void LevelPics::updateLevelCanvasPixmapArray()
{
  levelsprite_->setFrame(0);
}

//-------------------------------------------------------------
/*
 * draw a filled rectangle @levelmask_ with color over all pixels with fillcolor
 * 
 * @return true if something was painted
 */

bool LevelPics::drawRectLevel(int x, int y, int width, int height, QColor maskcolor, QColor maskfillcolor, QColor pixmapcolor)
{
  int painted=0;
  QColor tempmaskfillcolor;

  if (maskfillcolor==QColor(Qt::black))
    tempmaskfillcolor=QColor(Qt::yellow);
  else
    tempmaskfillcolor=maskfillcolor;

  if ((x>=0)&&(x<=getLevelWidth())&&(y>=0)&&(y<=getLevelHeight())&&(x+width<=getLevelWidth())&&(y+height<=getLevelHeight()))
  {
    QPainter painter(levelcanvaspixmap_);
    painter.setPen(pixmapcolor.rgb());

    for (int i=x;i<x+width;i++)
    {
      for (int j=y;j<y+height;j++)
      { 
	QRgb pixel = levelmask_->pixel(i, j);

	if ((QColor(pixel)==maskfillcolor.rgb())||(QColor(pixel)==tempmaskfillcolor.rgb()))
	{
	  levelmask_->setPixel(i, j, maskcolor.rgb());

	  if (pixmapcolor.isValid())
	    painter.drawPoint(i, j);

	  painted=1;
	}
      }
    }

    painter.end();
  }

  return(painted);
}

//-------------------------------------------------------------
/*
 * draw a filled circle @levelmask_ and @levelcanvaspixmap_ with color over all pixels with fillcolor
 * okok, it's an ellipse ;-)
 */

void LevelPics::drawCircleLevel(int x, int y, int radius, QColor maskcolor, QColor maskfillcolor, QColor pixmapcolor)
{
  float tempypos;
  float temp=((radius*2)+1);
  int xpos;
  int ypos;

  QPainter painter(levelcanvaspixmap_);
  painter.setPen(pixmapcolor.rgb());
 

  for (int i=0;i<=(radius)*2+1;i++)
  {
    tempypos=sqrt(sin((i*(22/7))/(temp)))*((temp-1)/2);

    for (int j=-((int)tempypos);j<=(int)tempypos;j++)
    {
      xpos=x+i;
      ypos=y+j;
      
      if ((xpos>=0)&&(ypos>=0)&&(xpos<=getLevelWidth()-1)&&(ypos<=getLevelHeight()-1))
      {
	QRgb pixel = levelmask_->pixel(xpos, ypos);
	
	if (QColor(pixel)==maskfillcolor.rgb())
	{
	  levelmask_->setPixel(xpos, ypos, maskcolor.rgb());
	  painter.drawPoint(xpos, ypos);
	}
      }
    }
  }
  painter.end();
}

//-------------------------------------------------------------
/*
 * find out if color is in the given area
 * 
 * @return true if color was found in the defined area
 */

bool LevelPics::isColorAtLevelMask(int x, int y, int width, int height, QColor color)
{
  if ((x>=0)&&(x<=getLevelWidth()-1)&&(y>=0)&&(y<=getLevelHeight()-1)&&(x+width<=getLevelWidth()-1)&&(y+height<=getLevelHeight()-1))
  {
    for (int i=x;i<x+width;i++)
    {
      for (int j=y;j<y+height;j++)
      { 
	QRgb pixel = levelmask_->pixel(i, j);
	
	if (QColor(pixel)==color.rgb())
	{
	  return(true);
	}
      }
    }
  }
  return(false);
}

//-------------------------------------------------------------
/*
 * find out if other colors are in given area
 * 
 * @return true if another color was found in the defined area
 */

bool LevelPics::isOtherColorAtLevelMask(int x, int y, int width, int height, QColor color1, QColor color2)
{
  if (!color2.isValid())
    color2=color1;

  if ((x>=0)&&(x<=getLevelWidth())&&(y>=0)&&(y<=getLevelHeight())&&(x+width<=getLevelWidth())&&(y+height<=getLevelHeight()))
  {
    for (int i=x;i<x+width;i++)
    {
      for (int j=y;j<y+height;j++)
      { 
	QRgb pixel = levelmask_->pixel(i, j);
	
	if ((QColor(pixel)!=color1.rgb())&&(QColor(pixel)!=color2.rgb()))
	{
	  return(true);
	}
      }
    }
  }
  return(false);
}
